(* $Id: menu_package.ml 530 2005-04-04 11:58:34Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

open Godi_types
open Printf


let help_str =
  "This menu shows the details of one available software package. Furthermore,\n" ^
  "you can select (letter 'b'=build) or deselect (letter 'r'=remove) the package\n" ^
  "individually. The function 'k'=keep means to keep the installed version,\n" ^
  "or to leave the package uninstalled, whatever applies.\n" ^
  "\n" ^
  "Some packages need to be configured. If so, you can enter the configuration\n" ^
  "dialog by pressing the letter 'c' (but it is usually entered automatically).\n" ^
  "Configuration means that 'make' variables must be set in order to give\n" ^
  "GODI hints where to find system libraries, or to enable special functions\n" ^
  "at compile time. These variables can be edited in the configuration dialog\n" ^
  "and they are written into the godi.conf file.\n" ^
  "\n" ^
  "Note that there are now special configuration packages 'conf-<name>' that\n" ^
  "configure the location of system libraries. These packages have some built-in\n" ^
  "intelligence to auto-configure themselves, but sometimes it is necessary to\n" ^
  "give them hints by setting variables. So if the compilation of a 'conf'\n" ^
  "package fails, you can try to set the variables.\n" ^
  "\n" ^
  "You can test a certain configuration (whether it has options or not) by\n" ^
  "pressing 't'. This invokes the configure script of the package. Note\n" ^
  "that this works only if all required packages are already installed."
;;


let disp_status_str =
  function
      `Keep -> "Keep"
    | `Build | `Build_dep _ -> "b=build from source and install"
    | `Remove | `Remove_dep _ -> "r=remove from installation"
    | `Add | `Add_dep _ -> "a=add from binary package"   (* Cannot happen *)
;;


let disp_status ip p =
  match p#user_status with
      `Keep -> ( match ip with
		     Some ip -> disp_status_str ip#user_status
		   | None    -> disp_status_str p#user_status
	       )
    | _     -> disp_status_str p#user_status
;;


let full_version p =
  let v = p#version in
  let r = p#pkgrevision in
  match r with
      0 -> v
    | _ -> v ^ "godi" ^ string_of_int r
;;


let lf_re = Str.regexp "\n";;


let display ?inst_pkg (mk_pg : unit -> Miniui.page) pkg =
  let pg = mk_pg () in

  let p = any_pkg pkg in
  let ip =
    match inst_pkg with
	Some ipkg -> Some(any_pkg ipkg)
      | None      -> None
  in

  let have_config =
    match pkg with
	`Source_pkg spkg ->
	  spkg # build_message <> "" || spkg # build_options <> []
      | _ ->
	  false
  in

  let need_config = ref false in

  let rec show() =
    pg # clear();

    Menu_aux.add_compact_head ~lmargin:6 pg ">>> Single Package <<<";

    pg # put_line (sprintf "Package:           %s" p#name);
    pg # put_line (sprintf "Status:            %s" (disp_status ip p));
    pg # put_line (sprintf "Installed version: %s"
		     (match ip with None -> "" | Some ip -> full_version ip));
    pg # put_line (sprintf "Available version: %s"
		     (match pkg with
			  `Source_pkg _ ->
			    (full_version p ^ " (from source)")
			| `Binary_pkg _ ->
			    (full_version p ^ " (binary)")
			| _ ->
			    ""));

    pg # put_line "";
    pg # put_line ("Comment: " ^ p#comment);

    pg # put_line "";
    pg # put_line "Description: ";

    let lines = Str.split lf_re p#description in
    List.iter pg#put_line lines;

    if p#dependencies <> [] || p#build_dependencies <> [] then (
      pg # put_line "";
      pg # put_line "This package is dependent on:";
      List.iter
	(fun d ->
	   pg # put_line (sprintf "- %s"
			    (Godi_versions.print_dependency d))
	)
	p#dependencies;
      List.iter
	(fun d ->
	   pg # put_line (sprintf "- %s (only for build-time)"
			    (Godi_versions.print_dependency d))
	)
	p#build_dependencies;

    );

    pg # put_line "";
    pg # put_line ("GODI maintainer: " ^ p#godi_maintainer);

    (* Add buttons: *)
    let need_keep = ref false in
    ( match pkg with
	  `Source_pkg _ ->
	    ( match p#user_status with
		  `Keep ->
		    ignore(pg # add_foot_button
			     ~text:"build&install" ~underline:0
			     ~callback:build (`Keypress 'b'));
		| `Build
		| `Build_dep _ ->
		    need_keep := true
		| _ ->
		    ()
	    )
	| `Binary_pkg _ ->
	    (* TODO *)
	    ()
	| `Installed_pkg _ ->
	    ()
    );
    ( match inst_pkg with
	  Some (`Installed_pkg ip) ->
	    ( match ip#user_status with
		  `Keep ->
		    ignore(pg # add_foot_button
			     ~text:"remove" ~underline:0
			     ~callback:remove (`Keypress 'r'));
		| `Remove
		| `Remove_dep _ ->
		    need_keep := true
		| _ ->
		    ()
	    )
	| Some _
	| None ->
	    ()
    );

    if !need_keep then
      ignore(pg # add_foot_button
	       ~text:"keep as it is" ~underline:0
	       ~callback:keep (`Keypress 'k'));

    if have_config then
      ignore(pg # add_foot_button
	       ~text:"configure" ~underline:0
	       ~callback:configure (`Keypress 'c'));

    if pkgtype pkg = `Source then
      ignore(pg # add_foot_button
	       ~text:"test configuration" ~underline:0
	       ~callback:(Menu_testconf.display pg#new_page pkg)
	       (`Keypress 't'));

    ignore(pg # add_foot_button ~text:"help" ~underline:0
             ~callback:(Menu_help.display pg#new_page help_str)
             (`Keypress 'h'));

    ignore(pg # add_foot_button ~text:"exit menu" ~underline:1
             ~callback:(fun () ->
			  if !need_config then configure();
			  pg#quit())
	    (`Keypress 'x'));


  and build() =
    (* Ensures that the configure dialog is shown; either the user
     * invokes it, or it is automatically called on 'x'
     *)
    ( match pkg with
	  `Source_pkg sp ->
	    sp # set_config_dialog_shown false;
	    need_config := have_config
	| _ ->
	    ()
    );
    (match ip with Some ip -> ip # set_user_status `Keep | None -> ());
    p # set_user_status `Build;
    show()

  and remove() =
    (* p == ip is possible! *)
    need_config := false;
    p # set_user_status `Keep;
    (match ip with
	 Some ip -> ip # set_user_status `Remove
       | None -> ()
    );
    show()

  and keep() =
    need_config := false;
    (match ip with Some ip -> ip # set_user_status `Keep | None -> ());
    p # set_user_status `Keep;
    show()

  and configure() =
    Menu_configure.display (pg#new_page()) pkg;
    need_config := false

  in

  show();
  pg # loop();
;;
